const Cc = Components.classes;
const Ci = Components.interfaces;

const NEW_LINE = "\r\n";

var gConsole = {
  get consoleService() {
    delete this.consoleService;
    return this.consoleService =
      Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
  },

  // amount of messages to hold in the console, corresponds to the services
  // buffer limit
  messageLimit: 250,

  get console() {
    delete this.console;
    return this.console = document.getElementById("console");
  },

  get bundle() {
    delete this.bundle;
    return this.bundle = document.getElementById("ConsoleBundle");
  },

  init: function console__init() {
    this.consoleService.registerListener(this.messageObserver);
    this.appendInitialItems();
    // mostly here for tests, this informs any listeners that the ui and this
    // object are fully initialized
    Cc["@mozilla.org/observer-service;1"]
      .getService(Ci.nsIObserverService)
      .notifyObservers(window, "error-console-ui-done", null);
    //XXX To be removed!
    initOldValues();
  },

  destroy: function console_destroy() {
    this.consoleService.unregisterListener(this.messageObserver);
  },

  /** Clear the console and notify the service with a null message if we haven't
   * notified it yet. Since at times we clear the console use the param aNotify
   * to indicate whether or not the console should be notified.
   * 
   * @param aNotify A boolean indicating whether or not the console service
   *                should be notified of this clear.
  */   
  clearConsole: function console__clearConsole(aNotify) {
    if (aNotify) {
      this.consoleService.logStringMessage(null);
      this.consoleService.reset();
    }
    this.console.clearSelection();
    var newConsole = this.console.cloneNode(false);
    this.console.parentNode.replaceChild(newConsole, this.console);
    this.console = newConsole;
  },

  messageObserver: {
    observe: function console__message__observe(aMessage) {
      gConsole.processMessage(aMessage);
    }
  },

  appendInitialItems: function console__appendInitialItems() {
    var out = {};
    this.consoleService.getMessageArray(out, {});
    // In case getMessageArray returns 0-length array as null we'll use an empty
    // array
    var messages = out.value || [];

    var limit = messages.length - this.messageLimit;
    if (limit < 0) limit = 0;

    // Checks if the console has ever been cleared
    var i;
    for (i = messages.length - 1; i >= limit; --i)
      if (!messages[i].message)
        break;

    // Populate with messages after latest "clear"
    while (++i < messages.length)
      this.processMessage(messages[i]);
  },

  processMessage: function console__processMessage(aMessage) {
    // See if it's a script error to get more info
    if (aMessage instanceof Ci.nsIScriptError) {
      this.appendItem(aMessage, "error");
    }
    // Is it a console message?
    else if (aMessage instanceof Ci.nsIConsoleMessage) {
      if (aMessage.message)
        this.appendItem(aMessage.message, "message");
      else // observed a null/"clear" message
        this.clearConsole();
    }
    else {
      // Give up and append the object itself as a string
      this.appendItem(aMessage, "object");
    }
  },

  appendItem: function console__appendItem(aMessage, aType) {
    const XULNS =
      "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    // Remove the first item if we have reached the message buffer limit
    if (this.console.childNodes.length > this.messageLimit)
      this.console.removeChild(this.console.firstChild);
    var item = document.createElementNS(XULNS, "richlistitem");
    var label = document.createElementNS(XULNS, "label");
    if (aType == "error") {
      // Is this error actually just a non-fatal warning?
      var warning = aMessage.flags & Ci.nsIScriptError.warningFlag != 0;

      var typeText = warning ? "warning" : "error";
      item.setAttribute("type", typeText);
      label.setAttribute("value", aMessage.errorMessage);
    }
    else if (aType == "message" || aType == "object") {
      item.setAttribute("type", "message");
      label.setAttribute("value", aMessage);
    }
    item.appendChild(label);
    this.console.appendChild(item);
  },

  setMode: function console__setMode(aName) {
    this.console.setAttribute("mode", aName);
  },
}

//XXX To be removed!
var gCodeToEvaluate, gEvaluator, gTextBoxEval;

function initOldValues()
{
  /* aMode can end up invalid if it set by an extension that replaces */
  /* mode and then it is uninstalled or disabled */
  var bc = document.getElementById("Console:mode" + gConsole.console.getAttribute("mode")) ||
           document.getElementById("Console:modeAll");
  bc.setAttribute("checked", true);
}
